<h3>Download</h3>
<p>Current version: <b>{{ version }}</b></p>

<p>Download matplotlib from the sourceforge <a
href="http://sourceforge.net/projects/matplotlib">project</a> page
(but first take a look at the <a href="{{ pathto('users/installing')
}}">installing</a> page).  Here's a summary of <a href="{{
pathto('users/whats_new') }}">what's new</a>. </p>

<p>There are several matplotlib addon <a href="{{
pathto('users/toolkits') }}">toolkits</a>, including the projection
and mapping toolkit
<a href="http://matplotlib.sf.net/basemap/doc/html">basemap</a>.</p>

<p>Please <a href="http://sourceforge.net/project/project_donations.php?group_id=80706">donate</a>
to support matplotlib development.</p>

<h3>Need help?</h3>

<p>Check the <a href="{{ pathto('users/index') }}">user</a> guide,
the <a href="{{ pathto('faq/index') }}">faq</a>, the <a href="{{
pathto('api/index') }}">api</a> docs,
<a href="http://www.nabble.com/matplotlib---users-f2906.html">archives</a>,
and join the matplotlib
mailing <a href="http://sourceforge.net/mail/?group_id=80706">lists</a>.
The <a href="{{ pathto('search') }}">search</a> tool searches all of
the documentation, including full text search of almost 300 complete
examples which exercise almost every corner of matplotlib.</p>

<p>You can file bugs, patches and feature requests on the
sourceforge
<a href="http://sourceforge.net/tracker2/?group_id=80706">tracker</a>,
but it is a good idea to ping us on the mailing list too.</p>

<p>For details on what's new, see the detailed <a href="{{
pathto('_static/CHANGELOG', 1) }}">changelog</a>.  Anything that could
require changes to your existing codes is logged in the <a href="{{
pathto('api/api_changes.html', 1) }}">api changes</a> file.</p>

<h3>Other stuff</h3>

<p>The matplotlib <a href="{{ pathto('users/license') }}">license</a>
is based on the Python Software Foundation
<a href="http://www.python.org/psf/license">(PSF)</a> license.</p>

<p>There is an active developer community and a long list of people
who have made significant <a href="{{ pathto('users/credits')
}}">contributions</a>.</p>

