'''Configuration values in CGATReport.

'''

import os

SEPARATOR = "@"

HTML_IMAGE_FORMAT = ('main', 'png', 80)
LATEX_IMAGE_FORMAT = ()  # ('pdf', 'pdf' 50)

ADDITIONAL_FORMATS = [
    ('hires', 'hires.png', 200),
    # ('eps', 'eps', 50),
    # ('svg', 'svg', 50),
    # ('pdf', 'pdf', 50),
]
